local AddonName, GGL = ...
--[[-----------------------------------------------------------------------------
Label Widget
Displays text and optionally an icon.
-------------------------------------------------------------------------------]]
local Type, Version = "GoldGoblinLootGUIWidget-QualityProgressBar", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, pairs = math.max, pairs
local tonumber = tonumber
local format = string.format

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent
local GetItemIcon, GetItemInfo = GetItemIcon, GetItemInfo

-- GGL
local L = GGL.Locale

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]


--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
    ["OnAcquire"] = function(self)
        self:SetQualityList()
        self:SetText()
        self:SetRightText()
        self:SetHeight(15)
	end,

    ["OnWidthSet"] = function(self, width)
        self:UpdateBars()
    end,

    --- format:
    -- {
    --      [qualityID] = count,
    --      itemsByQuality[qualityID] = count,
    --      itemCount[itemHyperLink] = itemCopperValue,
    -- }
    ["SetQualityList"] = function(self, list, isPercent)
        self.isPercent = isPercent

        local numItems = 0
        if isPercent then
            numItems = 100
            self.percentList = list
        else
            for i = 0, #GGL.ITEM_QUALITY_INFO do
                if self.progressTextures[i] and list and list[i] then
                    numItems = numItems + list[i]
                end
            end
            if self.noListUpdate then
                -- keep self.percentList
            elseif list then
                self.percentList = {}
                for i = 0, #GGL.ITEM_QUALITY_INFO do
                    if self.progressTextures[i] and list then
                        if list[i] and list[i] > 0 then
                            self.percentList[i] = (100 / numItems) * (list[i] or 0)
                        else
                            self.percentList[i] = 0
                        end
                    end
                end
            else
                self.percentList = nil
            end
        end
        self.list = list

        local frameWidth = self.progressFrame:GetWidth()-#GGL.ITEM_QUALITY_INFO
        local widthPercent = frameWidth / numItems
        for i = 1, #self.progressTextures do
            local t = self.progressTextures[i]
            if list and list[i] and list[i] > 0 then
                t:SetWidth(widthPercent * list[i])
            else
                t:SetWidth(1)
            end
        end

        if not isPercent and self.showValueText then
            self.valueText:SetText(numItems)
            self.valueTextBg:SetWidth(max(self.valueText:GetUnboundedStringWidth() + 6, self.minValueTextWidth))
            self.valueTextBg:Show()
        else
            self.valueTextBg:Hide()
        end

        self.sortValue = self.sortValue or numItems
    end,

    ["UpdateBars"] = function(self)
        if self.list then
            self.noListUpdate = true
            self:SetQualityList(self.list, self.isPercent)
            self.noListUpdate = false
        end
    end,

    ["SetHeight"] = function(self, height)
        self.frame:SetHeight(height)
    end,

    ["SetText"] = function(self, text)
        if text and text ~= "" then
            self.text:Show()
            self.text:SetText(text)
            self._text = text
        else
            self.text:Hide()
            self._text = nil
        end
        self:UpdateFrameAnchors()
        self:UpdateBars()
    end,

    ["SetRightText"] = function(self, text)
        if text and text ~= "" then
            self.textRight:Show()
            self.textRight:SetText(text)
            self._textRightWidth = self.textRight:GetUnboundedStringWidth()
            self._textRight = text
        else
            self.textRight:Hide()
            self._textRight = nil
        end
        self:UpdateFrameAnchors()
        self:UpdateBars()
    end,

    ["UpdateFrameAnchors"] = function(self)
        if self._text and self._textRight then
            self.progressFrame:SetPoint("LEFT", self.frame, "CENTER", 0, 0)
            self.text:SetPoint("RIGHT", self.progressFrame, "LEFT", -(self._textRightWidth + 5), 0)
        elseif self._text then
            self.progressFrame:SetPoint("LEFT", self.frame, "CENTER", 0, 0)
            self.text:SetPoint("RIGHT", self.progressFrame, "LEFT", 0, 0)
        elseif self._textRight then
            self.progressFrame:SetPoint("LEFT", self.frame, "CENTER", 0, 0)
        else
            self.progressFrame:SetPoint("LEFT", self.frame, 0, 0)
        end
    end,

    ["ShowValueText"] = function(self, state)
        if state == self.showValueText then return end
        self.showValueText = state
        self:UpdateBars()
    end,

    ["SetSortValue"] = function(self, value)
        self.sortValue = value or 0
    end,

    ["GetSortValue"] = function(self)
        return self.sortValue or 0
    end,
}

local TT_VALUE_FORMAT = "%d"
local TT_VALUE_PERCENT_FORMAT = "%d - %.1f%%"
local function Frame_OnEnter(self)
    if not self.obj then return end
    local obj = self.obj
    local tooltip = _G.GameTooltip
    local addEmptyLine = false
    tooltip:ClearLines()
    tooltip:SetOwner(UIParent,"ANCHOR_CURSOR")
    tooltip:SetText(obj._text or L["Item Count"])

    for i = 0, #obj.percentList do
        if obj.list[i] and obj.list[i] > 0 then
            if addEmptyLine and obj.list.itemsByQuality and obj.list.itemsByQuality[i] then
                tooltip:AddLine(" ")
            end
            tooltip:AddDoubleLine(GGL.ITEM_QUALITY_INFO[i].coloredDesc, obj.isPercent and obj.percentList[i].."%" or format(TT_VALUE_PERCENT_FORMAT, obj.list[i] or 0, obj.percentList[i] or 0), nil, nil, nil, 1, 1, 1)
            if obj.list.itemsByQuality and obj.list.itemsByQuality[i] then
                for j = 1, #obj.list.itemsByQuality[i] do
                    tooltip:AddDoubleLine(obj.list.itemCount[obj.list.itemsByQuality[i][j]], obj.list.itemsByQuality[i][j], 1, 1, 1)
                end
                addEmptyLine = true
            else
                addEmptyLine = false
            end
        end
    end
    tooltip:Show()
end

local function  Frame_OnLeave(self)
    _G.GameTooltip:Hide()
end

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
    local frame = CreateFrame("FRAME")
    frame:SetScript("OnEnter", Frame_OnEnter)
    frame:SetScript("OnLeave", Frame_OnLeave)

    local progressFrame = CreateFrame("FRAME", nil, frame)
    progressFrame:SetPoint("TOP", frame, 0, 0)
    progressFrame:SetPoint("LEFT", frame, 0, 0)
    progressFrame:SetPoint("RIGHT", frame, 0, 0)
    progressFrame:SetPoint("BOTTOM", frame, 0, 0)

    local progressTextures = {}
    for quality = #GGL.ITEM_QUALITY_INFO, 0, -1 do
        local qData = GGL.ITEM_QUALITY_INFO[quality]

        local progressTexture = progressFrame:CreateTexture(nil, "BACKGROUND")
        progressTexture:SetPoint("TOP", progressFrame, 0, 0)
        progressTexture:SetPoint("BOTTOM", progressFrame, 0, 0)
        if quality == #GGL.ITEM_QUALITY_INFO then
            progressTexture:SetPoint("RIGHT", progressFrame, 0, 0)
        else
            progressTexture:SetPoint("RIGHT", progressTextures[quality+1], "LEFT", 0, 0)
        end
        if quality == 0 then
            progressTexture:SetPoint("LEFT", progressFrame, 0, 0)
        end

        progressTexture:SetColorTexture(qData.color.r, qData.color.g, qData.color.b, 0.5)

        progressTextures[quality] = progressTexture
    end

    local gfhFontName, gfhFontHeight, gfhFontFlags = GameFontHighlightSmall:GetFont()
    local valueTextBg = GGL.GUI:CreateBoxFrame(progressFrame)
    valueTextBg:SetPoint("CENTER", progressFrame, 0, 0)
    --valueTextBg:SetBackdropBorderColor(1, 1, 1, 0.6)
    valueTextBg:SetHeight(gfhFontHeight + 2)

    local valueText = valueTextBg:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
    valueText:SetAllPoints(valueTextBg)
    valueText:SetJustifyH("CENTER")
    valueText:SetJustifyV("MIDDLE")
    valueText:SetText("9999")
    valueText:SetNonSpaceWrap(false)
    valueTextBg:SetWidth(valueText:GetUnboundedStringWidth() + 6)
    valueTextBg:Hide()

    local textRight = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    textRight:SetPoint("TOPRIGHT", progressFrame, "TOPLEFT", -3, 0)
    textRight:SetPoint("BOTTOMRIGHT", progressFrame, "BOTTOMLEFT", -3, 0)
    textRight:SetPoint("LEFT", frame, 0, 0)
    textRight:SetJustifyH("RIGHT")
    textRight:SetJustifyV("MIDDLE")
    textRight:SetNonSpaceWrap(false)
    textRight:Hide()

    local text = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
    text:SetPoint("TOPLEFT", frame, 0, 0)
    text:SetPoint("BOTTOMLEFT", frame, 0, 0)
    text:SetPoint("RIGHT", frame, "CENTER", 0, 0)
    text:SetJustifyH("LEFT")
    text:SetJustifyV("MIDDLE")
    text:Hide()

    local widget = {
        type  = Type,
        frame = frame,
        progressFrame = progressFrame,
        text = text,
        textRight = textRight,
        valueTextBg = valueTextBg,
        valueText = valueText,
        progressTextures = progressTextures,
        minValueTextWidth = valueTextBg:GetWidth(),
    }
    frame.obj = widget

    -- methodes
    for method, func in pairs(methods) do
        widget[method] = func
    end

    return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
