local AddonName, GGL = ...

local Templates = {}
GGL.Templates = Templates

-- Lua
local ipairs, pairs = ipairs, pairs
local assert = assert
local setmetatable = setmetatable
local table_remove = table.remove

-- local
local TemplateRegister = {}
local HelperTemplateProto = {}
local HelperTemplateProto_mt = { __index = HelperTemplateProto }

local TemplateProto = {
    ["GetType"] = function(self)
        return self.__type
    end,

    ["CallBaseFunction"] = function(self, base, name, ...)
        if self.base and self.base[base] and self.base[base][name] then
            return self.base[base][name](self, ...)
        end
    end,
}
local TemplateProto_mt = { __index = TemplateProto }

-- ##################
-- Templates
-- ##################

-- @return  proto
function Templates:Register(name, ...)
    assert(not TemplateRegister[name], "Template with name '"..name.."' already found.")

    local child = {...}
    local proto = {
        __type = name
    }
    setmetatable(proto, TemplateProto_mt)

    if #child > 0 then
        proto.base = proto.base or {} -- save all functions in base table if they get overridden
        for i, n in ipairs(child) do
            if TemplateRegister[n] then
                proto.base[n] = {}
                for k,v in pairs(TemplateRegister[n].proto) do
                    proto[k] = v
                    proto.base[n][k] = v
                end
            else
                error("Unknown template '"..n.."'")
            end
        end
    end

    local helper = {
        __name = name
    }
    setmetatable(helper, HelperTemplateProto_mt)

     TemplateRegister[name] = {
        proto = proto,
        mt = { __index = proto },
        helper = helper
    }

    return proto, helper
end

function Templates:New(name, ...)
    assert(TemplateRegister[name], "Template with name '"..name.."' not found.")

    local tempT = {}
    local proto = TemplateRegister[name].proto

    -- init
    setmetatable(tempT, TemplateRegister[name].mt)

    -- Constructor
    if tempT.Constructor then
        local new = tempT:Constructor(...)
        -- error
        if new == false then
            return
        end
        -- Override
        if new then
            return new
        end
    end

    return tempT
end

-- Try get the template or create a new one..
function Templates:Get(name, ...)
    assert(TemplateRegister[name], "Template with name '"..name.."' not found.")

    if TemplateRegister[name].Get then
        return TemplateRegister[name]:Get(...) or Templates:New(name, ...)
    else
        return Templates:New(name, ...)
    end
end

-- ##################
-- HelperTemplateProto
-- ##################
function HelperTemplateProto:AddOnInit(func)
    if func then
        GGL:AddInitFuncPlayerLogin(func)
    end
end

function HelperTemplateProto:GetType()
    return self.__name
end

function HelperTemplateProto:RegisterEvent(event)
    if not self.EventFrame then
        self.EventFrame = CreateFrame("FRAME")
        self.EventFrame:SetScript("OnEvent",
            function(_, event, ...)
                if self[event] then
                    self[event](self, ...)
                end
            end
        )
    end
    assert(self[event], "function for event '"..(event or "nil").."' not found")
    self.EventFrame:RegisterEvent(event)
end

function HelperTemplateProto:UnregisterEvent(event)
    if self.EventFrame then
        self.EventFrame:UnregisterEvent(event)
    end
end

function HelperTemplateProto:UnregisterAllEvents()
    if self.EventFrame then
        self.EventFrame:UnregisterAllEvents()
    end
end

GGL:AddCallbackHandler(HelperTemplateProto)